"""
This module holds system-global variables for the broker.

Created on Jan 21, 2016

@author: Nicklas Boerjesson
"""

__author__ = 'nibo'

# The states of the processes
states = []
states_lookup = {}

#: Logged in peers
peers = {}
